@@ -86,14 +86,16 @@ GtkWidget *pwin = NULL;
 
 void properties_dialog(AppletWidget *applet, gpointer data)
 {
+  static GnomeHelpMenuEntry help_entry = { NULL, "properties" };
   gchar *titles[2] = { "Continent/City" , NULL};
-
   GtkWidget *label;
   GtkWidget *frame;
   GtkWidget *hbox;
   GtkWidget *list;
   GtkWidget *sw;
   
+  help_entry.name = gnome_app_id;
+
   if(pwin) 
   {
     gdk_window_raise(pwin->window);
@@ -148,6 +150,10 @@ void properties_dialog(AppletWidget *applet, gpointer data)
   gtk_signal_connect( GTK_OBJECT(pwin),"apply", GTK_SIGNAL_FUNC(property_apply_cb), NULL );
   gtk_signal_connect( GTK_OBJECT(pwin),"destroy", GTK_SIGNAL_FUNC(property_destroy_cb), NULL );
 */
+  gtk_signal_connect( GTK_OBJECT(pwin), "help",
+		      GTK_SIGNAL_FUNC(gnome_help_pbox_display),
+		      &help_entry );
+  
   gtk_widget_show_all(pwin);
 
   enum_timezones(list);
