@@ -80,12 +80,13 @@ battery_update(gpointer data)
   battery_read_charge(&percentage, &ac_online, &hours_remaining,
 		      &minutes_remaining);
 
+/* seems to always get displayed, no idea who is defining DEBUG
 #ifdef DEBUG
   fprintf(stderr, "\tRead: %d%%, ac %s, %d hours %d minutes remaining\n",
 	  percentage, ac_online ? "online" : "offline", hours_remaining,
 	  minutes_remaining);
 #endif
-
+*/
   /*
    *
    * Graph Mode 
@@ -522,8 +523,8 @@ make_new_battery_applet(void)
   /* Nothing is drawn until this is set. */
   bat->setup = TRUE;
 
-  gtk_timeout_add(500, (GtkFunction) battery_update,  bat);
-
+  bat->graph_timeout_id = gtk_timeout_add(1000 * bat->graph_interval,
+					  (GtkFunction) battery_update, bat);
 } /* make_new_battery_applet */
 
 void
