@@ -4,10 +4,10 @@
 #include "battery.h"
 
 void battery_session_load(char * cfgpath, BatteryData * bat);
-void battery_session_save(GtkWidget * w,
-			  const char * cfgpath,
-			  const char * globcfgpath,
-			  gpointer data);
+int battery_session_save(GtkWidget * w,
+			 const char * cfgpath,
+			 const char * globcfgpath,
+			 gpointer data);
 void battery_session_defaults(BatteryData * bat);
 
 #endif /* _SESSION_H */
