@@ -133,7 +133,7 @@ apply_properties(Fish *fish)
 		g_free(tmp);
 
 		tmp = splice_name(label_format, fish->prop.name);
-		gtk_label_set(GTK_LABEL(fish->fortune_label), tmp);
+		gtk_label_set_text(GTK_LABEL(fish->fortune_label), tmp);
 		g_free(tmp);
 	}
 	
@@ -200,13 +200,14 @@ destroy_pb(GtkWidget *w, Fish *fish)
 static void 
 properties_dialog(AppletWidget *aw, gpointer data)
 {
+        static GnomeHelpMenuEntry help_entry = { "fish_applet", "properties" };
 	Fish *fish = data;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *w;
 	GtkWidget *e;
 	GtkAdjustment *adj;
-	
+
 	if(fish->pb) {
 		gtk_widget_show(fish->pb);
 		gdk_window_raise(fish->pb->window);
@@ -288,6 +289,9 @@ properties_dialog(AppletWidget *aw, gpointer data)
 			   GTK_SIGNAL_FUNC(apply_cb),fish);
 	gtk_signal_connect(GTK_OBJECT(fish->pb), "destroy",
 			   GTK_SIGNAL_FUNC(destroy_pb),fish);
+	gtk_signal_connect(GTK_OBJECT(fish->pb), "help",
+			   GTK_SIGNAL_FUNC(gnome_help_pbox_display),
+			   &help_entry);
 
 	gtk_widget_show_all(fish->pb);
 }
