@@ -18,6 +18,8 @@ GList              *task_dest = NULL;
 GtkWidget          *hold_box = NULL;
 GtkWidget          *main_box = NULL;
 GtkWidget          *task_table = NULL;
+GtkWidget          *prop_button = NULL;
+GtkWidget          *arrow_button = NULL;
 GdkPixmap          *p_1 = NULL, *p_2 = NULL, *p_3 = NULL;
 GdkPixmap          *m_1 = NULL, *m_2 = NULL, *m_3 = NULL;
 
@@ -32,7 +34,7 @@ typedef struct {
 	gint                show_tasks; /*bool*/
 	gint                show_pager; /*bool*/
 	gint                show_icons; /*bool*/
-	gint		    show_arrow;
+	gint		    show_arrow; /*bool*/
 	gint                fixed_tasklist; /*bool*/
 } Config;
 
@@ -58,6 +60,18 @@ redo_interface(void)
 
   for (i = 0; i < 32; i++)
     desk_widget[i] = NULL;
+
+  /*hackish way to keep these widgets around*/
+  if(prop_button->parent)
+	  gtk_container_remove(GTK_CONTAINER(prop_button->parent),
+			       prop_button);
+  if(arrow_button->parent)
+	  gtk_container_remove(GTK_CONTAINER(arrow_button->parent),
+			       arrow_button);
+  /*kill the arrow*/
+  if(GTK_BIN(arrow_button)->child)
+    gtk_widget_destroy(GTK_BIN(arrow_button)->child);
+
   if (main_box)
     gtk_widget_destroy(main_box);
   if (popbox)
@@ -71,7 +85,7 @@ redo_interface(void)
   blists_num = 0;
   flists = NULL;
   main_box = NULL;
-  
+
   switch (applet_orient) 
     {
      case ORIENT_UP:
@@ -253,9 +267,6 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 					    20, 
 					    (gfloat)gdk_screen_width(), 
 					    16, 16, 16 );
-  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-		     GTK_SIGNAL_FUNC(cb_adj), &o_config.max_task_width);
-  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
   label = gtk_label_new(_("Maximum width of horizontal task list"));
   gtk_widget_show(label);
   spin = gtk_spin_button_new(adj, 1, 0);
@@ -264,14 +275,14 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		   0, 1, 0, 1, GTK_FILL|GTK_EXPAND,0,0,0);
   gtk_table_attach(GTK_TABLE(table), spin, 
 		   1, 2, 0, 1, GTK_FILL|GTK_EXPAND,0,0,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.max_task_width);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
 
   adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.max_task_vwidth,
 					    4, 
 					    (gfloat)gdk_screen_width(), 
 					    4, 4, 4 );
-  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-		     GTK_SIGNAL_FUNC(cb_adj), &o_config.max_task_vwidth);
-  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
   label = gtk_label_new(_("Maximum width of vertical task list"));
   gtk_widget_show(label);
   spin = gtk_spin_button_new(adj, 1, 0);
@@ -280,12 +291,12 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		   0, 1, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
   gtk_table_attach(GTK_TABLE(table), spin, 
 		   1, 2, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.max_task_vwidth);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
 
   adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.task_rows_h, 1, 
 					    8, 1, 1, 1 );
-  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-		     GTK_SIGNAL_FUNC(cb_adj), &o_config.task_rows_h);
-  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
   label = gtk_label_new(_("Number of rows of horizontal tasks"));
   gtk_widget_show(label);
   spin = gtk_spin_button_new(adj, 1, 0);
@@ -294,11 +305,12 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		   0, 1, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
   gtk_table_attach(GTK_TABLE(table), spin, 
 		   1, 2, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.task_rows_h);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
   adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.task_rows_v, 1, 
 					    4, 1, 1, 1 );
-  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-		     GTK_SIGNAL_FUNC(cb_adj), &o_config.task_rows_v);
-  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
   label = gtk_label_new(_("Number of vertical columns of tasks"));
   gtk_widget_show(label);
   spin = gtk_spin_button_new(adj, 1, 0);
@@ -307,11 +319,12 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		   0, 1, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
   gtk_table_attach(GTK_TABLE(table), spin, 
 		   1, 2, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.task_rows_v);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+
   adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)config.pager_rows, 1, 
 					    8, 1, 1, 1 );
-  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_rows);
-  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
   label = gtk_label_new(_("Number rows of pagers"));
   gtk_widget_show(label);
   spin = gtk_spin_button_new(adj, 1, 0);
@@ -320,6 +333,9 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		   0, 1, 4, 5, GTK_FILL|GTK_EXPAND,0,0,0);
   gtk_table_attach(GTK_TABLE(table), spin, 
 		   1, 2, 4, 5, GTK_FILL|GTK_EXPAND,0,0,0);
+  gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+		     GTK_SIGNAL_FUNC(cb_adj), &o_config.pager_rows);
+  gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
 
   gtk_widget_show(prop);
 }
@@ -383,26 +399,6 @@ util_get_atom(Window win, gchar *atom, Atom type, gint *size)
   return NULL;
 }
 
-gchar              *
-util_reduce_chars(gchar * s, int num)
-{
-  int                 l;
-  gchar              *ss;
-  
-  l = strlen(s);
-  if (l > num)
-    {
-      ss = g_malloc(num + 4);
-      ss[0] = 0;
-      strncat(ss, s, num);
-      strcat(ss, "...");
-      return ss;
-    }
-  else
-    return strdup(s);
-}
-
-
 
 /* CLIENT WINDOW manipulation functions */
 
@@ -506,7 +502,7 @@ client_win_show(Task *t)
   if (!t->focused)
     XSetInputFocus(GDK_DISPLAY(), t->win, RevertToPointerRoot, CurrentTime);
   XRaiseWindow(GDK_DISPLAY(), t->win);
-  if ((!t->sticky) && (t->desktop != current_desk - 1) && (!t->iconified))
+  if ((!t->sticky) && (t->desktop != current_desk) && (!t->iconified))
     gnome_win_hints_set_current_workspace(t->desktop);
   if (!t->iconified)
     {
@@ -676,6 +672,12 @@ custom_popbox_show(GtkWidget * widget)
 int
 showpop_cb(GtkWidget *widget, gpointer data)
 {
+  if(!popbox)
+    {
+      create_popbox();
+      populate_tasks(TRUE);
+    }
+
   custom_popbox_show(popbox);
   /*XXX: return false even though clicked is a void signal, but all the _event's
     need a return value*/
@@ -710,7 +712,7 @@ cb_task_change(GtkWidget *widget, GdkEventProperty * ev, Task *t)
 	  desktop_draw(t->desktop);
 	  desktop_draw(tdesk);
 	  if (!config.tasks_all)
-	    populate_tasks();	  
+	    populate_tasks(FALSE);	  
 	}
       else
 	desktop_draw(t->desktop);
@@ -739,7 +741,7 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
       current_desk = desk;
       desktop_draw(pdesk);
       desktop_draw(current_desk);
-      populate_tasks();
+      populate_tasks(FALSE);
       return;
     }
   
@@ -749,7 +751,7 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
       tasks_changed = 0;
       tasks_update();
       if (tasks_changed)
-	populate_tasks();
+	populate_tasks(FALSE);
       tasks_changed = 0;
       return;
     }
@@ -760,7 +762,7 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
       num_desk = gnome_win_hints_get_workspace_count();
       if (num_desk < 1)
 	num_desk = 1;
-      populate_tasks();
+      populate_tasks(FALSE);
       redo_interface();
       return;
     }
@@ -798,7 +800,7 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
 	  g_free(da);
 	}
       if (!config.tasks_all)
-	populate_tasks();
+	populate_tasks(FALSE);
       desktop_draw(current_desk);
     }
   at = gdk_atom_intern(XA_WIN_WORKSPACE_NAMES, FALSE);
@@ -1007,9 +1009,7 @@ task_get_info(Task *t)
 	needredo = 1;
     }
   if ((psticky != t->sticky) || (needredo))
-    {
-      populate_tasks();
-    }
+    populate_tasks(FALSE);
 }
 
 gint
@@ -1099,7 +1099,8 @@ task_add(Window win)
       /*if the window is ours don't make it select out some events only!*/
       XWindowAttributes attr;
       XGetWindowAttributes(GDK_DISPLAY(), win, &attr);
-      XSelectInput(GDK_DISPLAY(), win, attr.your_event_mask | PropertyChangeMask |
+      XSelectInput(GDK_DISPLAY(), win,
+		   attr.your_event_mask | PropertyChangeMask |
 	           FocusChangeMask | StructureNotifyMask);
     }
 
@@ -1113,57 +1114,47 @@ task_add(Window win)
 
 
 void 
-task_delete(Window win)
+task_delete(Task *t)
 {
   guint               i, tdesk;
-  Task               *t;
   gchar               tstick;
-  GList              *ptr;
+  
+  g_return_if_fail(t!=NULL);
 
   gdk_error_warnings = 0;
-  ptr = tasks;
-  while(ptr)
+
+  tasks = g_list_remove(tasks, t);
+  tstick = t->sticky;
+  tdesk = t->desktop;
+  if (t->name)
+    g_free(t->name);
+  if (t->gdkwin)
+    gdk_window_unref(t->gdkwin);
+  if (t->frame_gdkwin)
+    gdk_window_unref(t->frame_gdkwin);
+  g_free(t);
+  if (!tstick)
+    desktop_draw(tdesk);
+  else
     {
-      t = ptr->data;
-      ptr = ptr->next;
-      if (t)
-	{
-	  if (win == t->win)
-	    {
-	      tasks = g_list_remove(tasks, t);
-	      tstick = t->sticky;
-	      tdesk = t->desktop;
-	      if (t->name)
-		g_free(t->name);
-	      if (t->gdkwin)
-		gdk_window_unref(t->gdkwin);
-	      g_free(t);
-	      if (!tstick)
-		desktop_draw(tdesk);
-	      else
-		{
-		  for (i = 0; i < (guint)num_desk; i++)
-		    desktop_draw(i);
-		}
-	      return;
-	    }
-	}
+      for (i = 0; i < (guint)num_desk; i++)
+        desktop_draw(i);
     }
 }
 
 Task *
 task_find(Window win)
 {
-  guint               n, i;
   Task               *t;
-
-  n = g_list_length(tasks);
-  for (i = 0; i < n; i++)
+  GList              *ptr;
+  
+  for(ptr = tasks;ptr!=NULL; ptr=g_list_next(ptr))
     {
-      t = (g_list_nth(tasks, i))->data;
+      t = ptr->data;
       if (win == t->win)
 	return t;
     }
+
   return NULL;
 }
 
@@ -1193,7 +1184,10 @@ tasks_match(Window * win, guint num)
 	  if (!there)
 	    {
 	      tasks_changed = 1;
-	      task_delete(t->win);
+	      /*XXX:although it will work in this case, removing stuff from
+		a list while we are going through it is nuts and asking for
+	        trouble*/
+	      task_delete(t);
 	    }
 	}
     }
@@ -1341,7 +1335,7 @@ main(int argc, char *argv[])
     {
       GtkWidget *d, *l;
       
-      d = gnome_ok_dialog(_("Gnome Pager Error"));
+      d = gnome_error_dialog(_("Gnome Pager Error"));
       l = gtk_label_new(_("You are not running a GNOME Compliant Window Manager.\n"
 			  "Please run a GNOME Compliant Window Manager.\n"
 			  "For Example:\n"
@@ -1349,12 +1343,9 @@ main(int argc, char *argv[])
 			  "Then start this applet again.\n"));
       gtk_widget_show(l);
       gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox), l, TRUE, TRUE, 5);
-      gtk_window_set_modal(GTK_WINDOW(d),TRUE);
-      gtk_window_set_position (GTK_WINDOW(d), GTK_WIN_POS_CENTER);
       gnome_dialog_run(GNOME_DIALOG(d));
-      gtk_widget_destroy(d);
       
-      exit(1);
+      gtk_exit(1);
     }
 
   /*FIXME: remove this later!!!!!, in favour of the WELL BEHAVED
@@ -1392,6 +1383,27 @@ main(int argc, char *argv[])
   if (!hold_box)
     {
       hold_box = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
+
+      main_box = gtk_hbox_new(0,0);
+      gtk_container_add(GTK_CONTAINER(hold_box), main_box);
+
+      /*XXX: a hackish way to bind events on these buttons so that
+	we will recieve the right click and such events over them to
+	display menus and allow dragging over them */
+      prop_button = gtk_button_new_with_label(_("?"));
+      gtk_widget_ref(prop_button);
+      gtk_widget_show(prop_button);
+      gtk_box_pack_start_defaults(GTK_BOX(main_box),prop_button);
+      arrow_button = gtk_button_new();
+      gtk_widget_ref(arrow_button);
+      gtk_widget_show(arrow_button);
+      gtk_box_pack_start_defaults(GTK_BOX(main_box),arrow_button);
+
+      gtk_signal_connect(GTK_OBJECT(prop_button), "clicked",
+			 GTK_SIGNAL_FUNC(cb_applet_properties), NULL);
+      gtk_signal_connect(GTK_OBJECT(arrow_button), "clicked",
+			 GTK_SIGNAL_FUNC(showpop_cb), NULL);
+
       applet_widget_add(APPLET_WIDGET(applet), hold_box);
       gtk_widget_show(hold_box);
     }
@@ -1591,7 +1603,7 @@ get_window_root_and_frame_id(Window w, Window *ret_frame, Window *ret_root)
 
 
 
-GList  *get_task_stacking(GList *tasks, gint desk)
+GList  *get_task_stacking(gint desk)
 {
   GList *tasks_on_desk=NULL;
   GList *root_ids=NULL;
@@ -1729,7 +1741,7 @@ actual_redraw(gpointer data)
     gtk_draw_vline(s, win,GTK_STATE_NORMAL, 2, 1 + h, 1 + ((x * w) / area_w));
   for (y = 1; y < area_h; y++)
     gtk_draw_hline(s, win,GTK_STATE_NORMAL, 2, 1 + w, 1 + ((y * h) / area_h));
-  stacking = get_task_stacking(tasks, desk);
+  stacking = get_task_stacking(desk);
   p = stacking;
   while (p)
     {
@@ -1886,8 +1898,8 @@ create_popbox(void)
 void
 init_applet_gui(int horizontal)
 {
-  GtkWidget *frame, *button, *arrow, *table;
-  GtkWidget *desk, *align, *box, *button2;
+  GtkWidget *frame, *arrow, *table;
+  GtkWidget *desk, *align, *box;
   gint i, j, k;
   
   if (main_box)
@@ -1937,61 +1949,44 @@ init_applet_gui(int horizontal)
 
   if (config.show_arrow) 
     {
-      button2 = gtk_button_new_with_label(_("?"));
-      gtk_widget_show(button2);
-      button = gtk_button_new();
-      gtk_widget_show(button);
+      if(horizontal)
+        box = gtk_vbox_new(FALSE, 0);
+      else
+        box = gtk_hbox_new(FALSE, 0);
+      gtk_widget_show(box);
+
       switch(applet_orient)
         {
 	 case ORIENT_UP:
 	  arrow = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_OUT);
-	  gtk_widget_show(arrow);
-	  gtk_container_add(GTK_CONTAINER(button), arrow);
-	  box = gtk_vbox_new(FALSE, 0);
-	  gtk_widget_show(box);
-	  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);	
-	  gtk_box_pack_start(GTK_BOX(box), button2, TRUE, TRUE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), arrow_button, FALSE, FALSE, 0);	
+	  gtk_box_pack_start(GTK_BOX(box), prop_button, TRUE, TRUE, 0);
 	  break;
 	 case ORIENT_DOWN:
 	  arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-	  gtk_widget_show(arrow);
-	  gtk_container_add(GTK_CONTAINER(button), arrow);
-	  box = gtk_vbox_new(FALSE, 0);
-	  gtk_widget_show(box);
-	  gtk_box_pack_start(GTK_BOX(box), button2, TRUE, TRUE, 0);
-	  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), prop_button, TRUE, TRUE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), arrow_button, FALSE, FALSE, 0);
 	  break;
 	 case ORIENT_LEFT:
 	  arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT);
-	  gtk_widget_show(arrow);
-	  gtk_container_add(GTK_CONTAINER(button), arrow);
-	  box = gtk_hbox_new(FALSE, 0);
-	  gtk_widget_show(box);
-	  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
-	  gtk_box_pack_start(GTK_BOX(box), button2, TRUE, TRUE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), arrow_button, FALSE, FALSE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), prop_button, TRUE, TRUE, 0);
 	  break;
 	 case ORIENT_RIGHT:
 	  arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
-	  gtk_widget_show(arrow);
-	  gtk_container_add(GTK_CONTAINER(button), arrow);
-	  box = gtk_hbox_new(FALSE, 0);
-	  gtk_widget_show(box);
-	  gtk_box_pack_start(GTK_BOX(box), button2, TRUE, TRUE, 0);
-	  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), prop_button, TRUE, TRUE, 0);
+	  gtk_box_pack_start(GTK_BOX(box), arrow_button, FALSE, FALSE, 0);
 	  break;
 	 default:
-	  arrow = NULL; /*shut up warning*/
+	  arrow = box = NULL; /*shut up warning*/
 	  g_assert_not_reached();
 	  break;
 	}
-      if (arrow)
-	{
-	  gtk_box_pack_start(GTK_BOX(main_box), box, FALSE, FALSE, 0);
-	  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			     GTK_SIGNAL_FUNC(showpop_cb), NULL);
-	  gtk_signal_connect(GTK_OBJECT(button2), "clicked",
-			     GTK_SIGNAL_FUNC(cb_applet_properties), NULL);
-	}
+
+     gtk_widget_show(arrow);
+     gtk_container_add(GTK_CONTAINER(arrow_button), arrow);
+
+     gtk_box_pack_start(GTK_BOX(main_box), box, FALSE, FALSE, 0);
     }
   
   frame = gtk_frame_new(NULL);
@@ -2020,10 +2015,8 @@ init_applet_gui(int horizontal)
     gtk_container_add(GTK_CONTAINER(frame), table);
   task_table = table;
   
-  create_popbox();
-
   emtpy_task_widgets();
-  populate_tasks();
+  populate_tasks(FALSE);
 }
 
 gboolean
@@ -2128,12 +2121,6 @@ emtpy_task_widgets(void)
   if (task_dest)
     g_list_free(task_dest);
   task_dest = NULL;
-  if (task_dest)
-    g_list_free(task_dest);
-  task_dest = NULL;
-  if (task_dest)
-    g_list_free(task_dest);
-  task_dest = NULL;
 }
 
 void
@@ -2186,8 +2173,7 @@ set_task_info_to_button(Task *t)
 {
   GtkWidget *button, *icon1, *icon2, *icon3, *label;
   GtkRequisition  req;
-  gint num, len, mw;
-  gchar *str;
+  gint num, mw;
   GList *p;
 
   if (!t)
@@ -2275,51 +2261,103 @@ set_task_info_to_button(Task *t)
        if (num < config.task_rows_h) 
 	 num = config.task_rows_h;
       mw = config.max_task_width / ((num + config.task_rows_h - 1) / config.task_rows_h);
-      if (t->name)
-	len = strlen(t->name);
-      else
-	len = strlen("???");
-      while ((req.width > mw) && (len > 0))
-	{
-	  len--;
-	  if (t->name)
-	    str = util_reduce_chars(t->name, len);
-	  else
-	    str = util_reduce_chars("???", len);
-	  if (label)
-	    gtk_label_set_text (GTK_LABEL(label), str);
-	  g_free(str);
-	  gtk_widget_size_request(button, &req);
-	}
-      if ((len <=0) && (label))
-	gtk_label_set_text (GTK_LABEL(label), "");
     }
   else
     {
       mw = config.max_task_vwidth;
-      if (t->name)
-	len = strlen(t->name);
-      else
-	len = strlen("???");
-      while ((req.width > mw) && (len > 0))
-	{
-	  len--;
-	  if (t->name)
-	    str = util_reduce_chars(t->name, len);
-	  else
-	    str = util_reduce_chars("???", len);
-	  if (label)
-	    gtk_label_set_text (GTK_LABEL(label), str);
-	  g_free(str);
-	  gtk_widget_size_request(button, &req);
-	}
-      if ((len <=0) && (label))
-	gtk_label_set_text (GTK_LABEL(label), "");
+    }
+  
+    /*if the size is too much and we can reduce it, do it*/
+    /*FIXME: this  won't work with i18n (with multibyte characters)
+      properly*/
+    if(req.width > mw && label && (!t->name || *t->name)) {
+	    int len;
+	    char *str;
+	    int button_width;
+
+	    gtk_label_set_text (GTK_LABEL(label), "");
+	    gtk_widget_size_request(button, &req);
+
+	    if (t->name)
+		    len = strlen(t->name);
+	    else
+		    len = 3; /*strlen("???");*/
+	    
+	    len--;
+	    str = g_malloc(len+4);
+	    if(t->name)
+		    strcpy(str,t->name);
+	    else
+		    strcpy(str,"???");
+	    strcat(str,"..");
+	    for(;len>0;len--) {
+		    int label_len;
+		    str[len]='.';
+		    str[len+3]='\0';
+
+		    label_len = gdk_string_width
+			    (GTK_WIDGET (label)->style->font, str);
+		    if(label_len+req.width<=mw)
+			    break;
+	    }
+	    if(len<=0)
+		    gtk_label_set_text (GTK_LABEL(label), "");
+	    else
+		    gtk_label_set_text (GTK_LABEL(label), str);
+	    g_free(str);
     }
 }
 
+static gint
+popup_button_pressed (GtkWidget *widget, GdkEventButton *event, Task *t)
+{
+	GtkWidget *popup;
+	if (event->button != 3)
+		return FALSE;
+	
+	popup = gtk_object_get_data(GTK_OBJECT(widget),"popup");
+	if(!popup) {
+		GtkWidget *item;
+		popup = gtk_menu_new();
+
+		item = gtk_menu_item_new_with_label(_("Show / Hide"));
+		gtk_widget_show(item);
+		gtk_menu_append(GTK_MENU(popup),item);
+		gtk_signal_connect(GTK_OBJECT(item),"activate",
+				   GTK_SIGNAL_FUNC(cb_showhide),t);
+
+		item = gtk_menu_item_new_with_label(_("Shade / Unshade"));
+		gtk_widget_show(item);
+		gtk_menu_append(GTK_MENU(popup),item);
+		gtk_signal_connect(GTK_OBJECT(item),"activate",
+				   GTK_SIGNAL_FUNC(cb_shade),t);
+
+		item = gtk_menu_item_new_with_label(_("Close"));
+		gtk_widget_show(item);
+		gtk_menu_append(GTK_MENU(popup),item);
+		gtk_signal_connect(GTK_OBJECT(item),"activate",
+				   GTK_SIGNAL_FUNC(cb_kill),t);
+
+		item = gtk_menu_item_new_with_label(_("Nuke"));
+		gtk_widget_show(item);
+		gtk_menu_append(GTK_MENU(popup),item);
+		gtk_signal_connect(GTK_OBJECT(item),"activate",
+				   GTK_SIGNAL_FUNC(cb_nuke),t);
+
+		gtk_signal_connect_object(GTK_OBJECT(widget),"destroy",
+					  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+					  GTK_OBJECT(popup));
+
+		gtk_object_set_data(GTK_OBJECT(widget),"popup",popup);
+	}
+	gnome_popup_menu_do_popup (popup, NULL, NULL, event, NULL);
+
+	return TRUE;
+}
+
+
 void
-populate_tasks(void)
+populate_tasks(int just_popbox)
 {
    Task            *t;
    GList           *p;
@@ -2327,19 +2365,31 @@ populate_tasks(void)
 #ifdef ANIMATION
    GtkWidget       *f1, *f2, *f3, *f4;
 #endif  
-   gint             n_rows, n_cols, num, i, j, k, mw, len;
-   gchar           *str;
-   GnomeUIInfo     uinfo[5] = {GNOMEUIINFO_END, GNOMEUIINFO_END, 
-      GNOMEUIINFO_END, GNOMEUIINFO_END};
+   gint             n_rows, n_cols, num, i, j, k;
 
   if (!task_table)
     return;
 
-  desktroy_task_widgets();
+  if (!just_popbox && popbox && !GTK_WIDGET_VISIBLE(popbox))
+    {
+      gtk_widget_destroy(popbox);
+      popbox_q = popbox = NULL;
+      g_list_free(task_dest);
+      task_dest = NULL;
+    }
 
+  if (just_popbox)
+    {
+      g_list_foreach(task_dest,(GFunc)gtk_widget_destroy,NULL);
+      g_list_free(task_dest);
+      task_dest = NULL;
+    }
+  else
+    desktroy_task_widgets();
+  
   if (!tasks)
     return;
-
+  
   j = 0;
   k = 0;
   n_rows = 0;
@@ -2358,15 +2408,8 @@ populate_tasks(void)
 	   )
 	  )
 	num++;
-      p = p->next;
-    }
-  p = tasks;
-  while (p)
-    {
-      t = (Task *)p->data;
-      
       if (popbox)
-	{
+        {
 	  button = gtk_button_new_with_label(t->name ? t->name : "");
 	  gtk_widget_show(button);
 	  gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
@@ -2383,18 +2426,32 @@ populate_tasks(void)
 		   (t->desktop >= 0) && (t->desktop < num_desk) && 
 		   (num_desk > 0) && (blists[(t->desktop % num_desk) + 1]))
 	    gtk_box_pack_start(GTK_BOX(blists[(t->desktop % num_desk) + 1]), 
-			       button, TRUE, TRUE, 0);
-	  task_dest = g_list_append(task_dest, button);
+				     button, TRUE, TRUE, 0);
 	  gtk_object_set_data(GTK_OBJECT(button), "task", t);
+	  task_dest = g_list_append(task_dest, button);
 	}
       
-      if (((!(config.tasks_all)) && 
-	   (t->desktop == current_desk)
-	   && (t->ax == area_x)
-	   && (t->ay == area_y)
+      p = p->next;
+    }
+  if (popbox_q)
+    gtk_widget_queue_resize(popbox_q);
+
+  if(just_popbox)
+    return;
+
+  p = tasks;
+  while (p)
+    {
+      t = (Task *)p->data;
+      
+      if ((((!(config.tasks_all)) && 
+	    (t->desktop == current_desk)
+	    && (t->ax == area_x)
+	    && (t->ay == area_y)
 	   )
-	  || (config.tasks_all) ||
-	  (t->sticky))
+	   || (config.tasks_all) ||
+	   (t->sticky)) &&
+	  !just_popbox)
 	{
 	  hbox = gtk_hbox_new(0, FALSE);
 	  gtk_widget_show(hbox);
@@ -2453,58 +2510,10 @@ populate_tasks(void)
 	  gtk_tooltips_set_tip(GTK_TOOLTIPS(tip), button, t->name, NULL);
 	  gtk_tooltips_enable(GTK_TOOLTIPS(tip));
 */	  
-	   uinfo[0].type            = GNOME_APP_UI_ITEM;
-	   uinfo[0].label           = N_("Show / Hide");
-	   uinfo[0].hint            = NULL;
-	   uinfo[0].moreinfo        = cb_showhide;
-	   uinfo[0].user_data       = t;
-	   uinfo[0].unused_data     = NULL;
-	   uinfo[0].pixmap_type     = GNOME_APP_PIXMAP_NONE;
-	   uinfo[0].pixmap_info     = GNOME_STOCK_MENU_OPEN;
-	   uinfo[0].accelerator_key = 0;
-	   uinfo[0].ac_mods         = (GdkModifierType) 0;
-	   uinfo[0].widget          = NULL;
-	   
-	   uinfo[1].type            = GNOME_APP_UI_ITEM;
-	   uinfo[1].label           = N_("Shade / Unshade");
-	   uinfo[1].hint            = NULL;
-	   uinfo[1].moreinfo        = cb_shade;
-	   uinfo[1].user_data       = t;
-	   uinfo[1].unused_data     = NULL;
-	   uinfo[1].pixmap_type     = GNOME_APP_PIXMAP_NONE;
-	   uinfo[1].pixmap_info     = GNOME_STOCK_MENU_OPEN;
-	   uinfo[1].accelerator_key = 0;
-	   uinfo[1].ac_mods         = (GdkModifierType) 0;
-	   uinfo[1].widget          = NULL;
-	   
-	   uinfo[2].type            = GNOME_APP_UI_ITEM;
-	   uinfo[2].label           = N_("Close");
-	   uinfo[2].hint            = NULL;
-	   uinfo[2].moreinfo        = cb_kill;
-	   uinfo[2].user_data       = t;
-	   uinfo[2].unused_data     = NULL;
-	   uinfo[2].pixmap_type     = GNOME_APP_PIXMAP_NONE;
-	   uinfo[2].pixmap_info     = GNOME_STOCK_MENU_OPEN;
-	   uinfo[2].accelerator_key = 0;
-	   uinfo[2].ac_mods         = (GdkModifierType) 0;
-	   uinfo[2].widget          = NULL;
-	   
-	   uinfo[3].type            = GNOME_APP_UI_ITEM;
-	   uinfo[3].label           = N_("Nuke");
-	   uinfo[3].hint            = NULL;
-	   uinfo[3].moreinfo        = cb_nuke;
-	   uinfo[3].user_data       = t;
-	   uinfo[3].unused_data     = NULL;
-	   uinfo[3].pixmap_type     = GNOME_APP_PIXMAP_NONE/*GNOME_APP_PIXMAP_STOCK*/;
-	   uinfo[3].pixmap_info     = GNOME_STOCK_MENU_OPEN;
-	   uinfo[3].accelerator_key = 0;
-	   uinfo[3].ac_mods         = (GdkModifierType) 0;
-	   uinfo[3].widget          = NULL;
-
-/* WARNING WARNING: FIXME FIXME:  gnome_menu_popup LEAKS like a sieve */
-	  menu = gnome_popup_menu_new(uinfo);
-	  gnome_popup_menu_attach(menu, button, t);
-/*	  task_dest = g_list_append(task_dest, menu);*/
+	  
+	  gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
+			      (GtkSignalFunc) popup_button_pressed,
+			      t);
 
 	  gtk_object_set_data(GTK_OBJECT(button), "icon1", icon1);
 	  gtk_object_set_data(GTK_OBJECT(button), "icon2", icon2);
@@ -2554,11 +2563,6 @@ populate_tasks(void)
       p = p->next;
     }
   gtk_table_resize(GTK_TABLE(task_table), n_rows, n_cols);
-  if (popbox_q)
-    {
-      GtkRequisition req;
-      
-      gtk_widget_size_request(popbox_q, &req);
-      gtk_widget_queue_resize(popbox_q);
-    }
+  if(config.fixed_tasklist && task_table->parent)
+	  gtk_widget_queue_resize(task_table->parent);
 }
