@@ -12,17 +12,25 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <net/if.h>
+
+#ifdef __OpenBSD__
+#  include <net/bpf.h>
+#  include <net/if_pppvar.h>
+#  include <net/if_ppp.h>
+#endif /* __OpenBSD__ */
+
 #ifdef __FreeBSD__
-#include <net/if_ppp.h>
+#    include <net/if_ppp.h>
 #endif /* __FreeBSD__ */
+
 #include <net/ppp_defs.h>
 
 #include <gnome.h>
 #include <applet-widget.h>
 
-#define MODEMLIGHTS_APPLET_VERSION_MAJ 0
-#define MODEMLIGHTS_APPLET_VERSION_MIN 99
-#define MODEMLIGHTS_APPLET_VERSION_REV 0
+#define MODEMLIGHTS_APPLET_VERSION_MAJ 1
+#define MODEMLIGHTS_APPLET_VERSION_MIN 0
+#define MODEMLIGHTS_APPLET_VERSION_REV 1
 
 extern gint UPDATE_DELAY;
 extern gchar *lock_file;
