@@ -31,7 +31,8 @@
 /* Toolbar pixmaps */
 #include "contents.xpm"
 
-#define IMAGE_TEMPFILE "/tmp/gnome-help-browser.tmpfile"
+#define DEFAULT_HEIGHT 500
+#define DEFAULT_WIDTH  600
 
 struct _helpWindow {
     /* Main app widget */
@@ -708,6 +709,9 @@ helpWindowNew(gchar *name,
 
 	/* make the help window */
 	w->helpWidget = gnome_helpwin_new();
+	gtk_xmhtml_set_anchor_underline_type(GTK_XMHTML(w->helpWidget),
+					    GTK_ANCHOR_SINGLE_LINE);
+	gtk_xmhtml_set_anchor_buttons(GTK_XMHTML(w->helpWidget), FALSE);
 	gtk_widget_show(w->helpWidget);
 
 	/* add a status bar */
@@ -741,7 +745,8 @@ helpWindowNew(gchar *name,
 	if (width && height)
 		gtk_widget_set_usize(GTK_WIDGET(w->app), width, height); 
 	else
-		gtk_widget_set_usize(GTK_WIDGET(w->app), 600, 450); 
+		gtk_widget_set_usize(GTK_WIDGET(w->app), 
+				     DEFAULT_WIDTH, DEFAULT_HEIGHT); 
 
 	if (x != y)
 		gtk_widget_set_uposition(GTK_WIDGET(w->app), x, y);
