@@ -667,6 +667,10 @@ add_menu_to_panel (GtkWidget *widget, void *data)
 	if(g_file_exists("/etc/X11/wmconfig"))
 		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
 
+	/*guess debian menus*/
+	if (g_file_exists("/etc/menu-methods/gnome"))
+		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
+
 	if(mf)
 		load_menu_applet(mf->menudir,flags, current_panel, 0);
 	else
@@ -919,7 +923,9 @@ show_item_menu(GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 	     menus as they are auto generated!*/
 	   !strstr(sim->item_loc,"/.gnome/apps-redhat/") &&
 	   access(sim->item_loc,W_OK)==0) {
+#ifdef PANEL_DEBUG
 		puts(sim->item_loc);
+#endif
 		/*file exists and is writable, we're in bussines*/
 		gtk_widget_set_sensitive(sim->prop_item,TRUE);
 	} else if(!sim->item_loc || errno==ENOENT) {
@@ -1412,13 +1418,59 @@ check_and_reread_applet(Menu *menu,int main_menu)
 	}
 }
 
-static int
-reposition_menu(gpointer data)
+/* XXX: hmmm stolen GTK code, the gtk_menu_reposition only calls
+   gtk_menu_position if the widget is drawable, but that's not the
+   case when we want to do it*/
+static void
+gtk_menu_position (GtkMenu *menu)
 {
-	gtk_menu_reposition(GTK_MENU(data));
-	return FALSE;
+  GtkWidget *widget;
+  GtkRequisition requisition;
+  gint x, y;
+ 
+  g_return_if_fail (menu != NULL);
+  g_return_if_fail (GTK_IS_MENU (menu));
+
+  widget = GTK_WIDGET (menu);
+
+  gdk_window_get_pointer (NULL, &x, &y, NULL);
+
+  /* We need the requisition to figure out the right place to
+   * popup the menu. In fact, we always need to ask here, since
+   * if one a size_request was queued while we weren't popped up,
+   * the requisition won't have been recomputed yet.
+   */
+  gtk_widget_size_request (widget, &requisition);
+      
+  if (menu->position_func)
+    (* menu->position_func) (menu, &x, &y, menu->position_func_data);
+  else
+    {
+      gint screen_width;
+      gint screen_height;
+      
+      screen_width = gdk_screen_width ();
+      screen_height = gdk_screen_height ();
+	  
+      x -= 2;
+      y -= 2;
+      
+      if ((x + requisition.width) > screen_width)
+	x -= ((x + requisition.width) - screen_width);
+      if (x < 0)
+	x = 0;
+      if ((y + requisition.height) > screen_height)
+	y -= ((y + requisition.height) - screen_height);
+      if (y < 0)
+	y = 0;
+    }
+  
+  gtk_widget_set_uposition (GTK_MENU_SHELL (menu)->active ?
+			        menu->toplevel : menu->tearoff_window, 
+			    x, y);
 }
 
+
 static void
 submenu_to_display(GtkWidget *menuw, GtkMenuItem *menuitem)
 {
@@ -1470,9 +1522,7 @@ submenu_to_display(GtkWidget *menuw, GtkMenuItem *menuitem)
 		}
 		g_slist_free(mfl);
 
-		/*we run this as an idle so that it runs after the
-		  show signal finished*/
-		gtk_idle_add(reposition_menu,menuw);
+		gtk_menu_position(GTK_MENU(menuw));
 	}
 }
 
@@ -1767,7 +1817,8 @@ destroy_menu (GtkWidget *widget, gpointer data)
 						     MENU_PROPERTIES);
 	if(prop_dialog)
 		gtk_widget_destroy(prop_dialog);
-	gtk_widget_destroy(menu->menu);
+	if(menu->menu)
+		gtk_widget_destroy(menu->menu);
 	g_free(menu->path);
 	g_free(menu);
 }
@@ -2018,6 +2069,29 @@ create_user_menu(char *title, char *dir, GtkWidget *menu, int fake_submenus,
 	return menu;
 }
 
+static GtkWidget *
+create_debian_menu(GtkWidget *menu, int fake_submenus, int fake)
+{
+	char *menudir = "/var/lib/gnome/Debian/.";
+
+	if (g_file_exists (menudir)) {
+		if (!fake || menu) {
+			menu = create_menu_at (menu, menudir, FALSE,
+					       _("Debian menus"), NULL,
+					       fake_submenus, FALSE);
+		} else {
+			menu = create_fake_menu_at (menudir, FALSE,
+						    _("Debian menus"), NULL);
+		}
+		/*this would be bad*/
+		g_return_val_if_fail (menu, NULL);
+	} else {
+		g_warning (_("No Debian menus found!"));
+	}
+
+	return menu;
+}
+
 GtkWidget *
 create_panel_root_menu(GtkWidget *panel)
 {
@@ -2027,47 +2101,60 @@ create_panel_root_menu(GtkWidget *panel)
 
 	panel_menu = gtk_menu_new();
 
-	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("System menus"));
-	gtk_menu_append (GTK_MENU (panel_menu), menuitem);
 	menu = create_system_menu(NULL,TRUE,TRUE);
 	if(menu) {
+		menuitem = gtk_menu_item_new ();
+		setup_menuitem (menuitem, 0, _("System menus"));
+		gtk_menu_append (GTK_MENU (panel_menu), menuitem);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
 		gtk_signal_connect(GTK_OBJECT(menu),"show",
 				   GTK_SIGNAL_FUNC(submenu_to_display),
 				   menuitem);
 	}
 
-	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("User menus"));
-	gtk_menu_append (GTK_MENU (panel_menu), menuitem);
 	menu = create_user_menu(_("User menus"),"apps",NULL,TRUE,TRUE,TRUE);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
-	gtk_signal_connect(GTK_OBJECT(menu),"show",
-			   GTK_SIGNAL_FUNC(submenu_to_display),
-			   menuitem);
+	if(menu) {
+		menuitem = gtk_menu_item_new ();
+		setup_menuitem (menuitem, 0, _("User menus"));
+		gtk_menu_append (GTK_MENU (panel_menu), menuitem);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
+		gtk_signal_connect(GTK_OBJECT(menu),"show",
+				   GTK_SIGNAL_FUNC(submenu_to_display),
+				   menuitem);
+	}
 
 	if(g_file_exists("/etc/X11/wmconfig")) {
-		menuitem = gtk_menu_item_new ();
-		setup_menuitem (menuitem, 0, _("Red Hat menus"));
-		gtk_menu_append (GTK_MENU (panel_menu), menuitem);
 		menu = create_user_menu(_("Red Hat menus"),"apps-redhat",
 					NULL,TRUE,TRUE,TRUE);
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
-		gtk_signal_connect(GTK_OBJECT(menu),"show",
-				   GTK_SIGNAL_FUNC(rh_submenu_to_display),
-				   menuitem);
-		gtk_signal_connect(GTK_OBJECT(menu),"show",
-				   GTK_SIGNAL_FUNC(submenu_to_display),
-				   menuitem);
-		/*gtk_signal_connect(GTK_OBJECT(menuitem),"enter_notify_event",
-				   GTK_SIGNAL_FUNC(rh_submenu_to_display),
-				   NULL);
-		gtk_signal_connect(GTK_OBJECT(menuitem),"enter_notify_event",
-				   GTK_SIGNAL_FUNC(submenu_to_display),
-				   NULL);*/
+		if(menu) {
+			menuitem = gtk_menu_item_new ();
+			setup_menuitem (menuitem, 0, _("Red Hat menus"));
+			gtk_menu_append (GTK_MENU (panel_menu), menuitem);
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
+						   menu);
+			gtk_signal_connect(GTK_OBJECT(menu),"show",
+					   GTK_SIGNAL_FUNC(rh_submenu_to_display),
+					   menuitem);
+			gtk_signal_connect(GTK_OBJECT(menu),"show",
+					   GTK_SIGNAL_FUNC(submenu_to_display),
+					   menuitem);
+		}
 	}
 
+	if(g_file_exists("/etc/menu-methods/gnome")) {
+		menu = create_debian_menu(NULL,TRUE,TRUE);
+		if(menu) {
+			menuitem = gtk_menu_item_new ();
+			setup_menuitem (menuitem, 0, _("Debian menus"));
+			gtk_menu_append (GTK_MENU (panel_menu), menuitem);
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
+			gtk_signal_connect(GTK_OBJECT(menu),"show",
+					   GTK_SIGNAL_FUNC(submenu_to_display),
+					   menuitem);
+		}
+	}
+
+
 	menuitem = gtk_menu_item_new();
 	gtk_menu_append(GTK_MENU(panel_menu), menuitem);
 	gtk_widget_show(menuitem);
@@ -2184,13 +2271,15 @@ convert_to_panel(GtkWidget *w, gpointer data)
 	for(i=0;current_panel->applet_list!=NULL;i++) {
 		AppletData *ad = current_panel->applet_list->data;
 		panel_widget_reparent(current_panel,newpanel,ad->applet,i);
+#ifdef PANEL_DEBUG
 		printf("%lX, %d\n",(long)ad->applet,i);
+#endif
 	}
 	gtk_widget_destroy(pw);
 }
 
-static void
-show_x_on_panels(GtkWidget *menu, gpointer data)
+void
+show_x_on_panels(GtkWidget *menu)
 {
 	GtkWidget *pw;
 	GtkWidget *convcorn = gtk_object_get_data(GTK_OBJECT(menu),"convcorn");
@@ -2254,17 +2343,6 @@ make_panel_submenu (GtkWidget *menu, int fake_submenus)
 			   GINT_TO_POINTER(SNAPPED_PANEL));
 	gtk_object_set_data(GTK_OBJECT(menu),"convsnap",menuitem);
 
-	/*actually show hide right now to get the first size
-	  request correctly, still has a display problem when
-	  main menu is dragged onto a different type of a panel,
-	  but the problem is minimal (it only happens if the person
-	  uses the same menu)*/
-	show_x_on_panels(menu,NULL);
-	
-	gtk_signal_connect(GTK_OBJECT(menu),"show",
-			   GTK_SIGNAL_FUNC(show_x_on_panels),NULL);
-
-
 	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Add main menu"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
@@ -2358,6 +2436,10 @@ add_special_entries (GtkWidget *menu, int fake_submenus)
 	setup_menuitem (menuitem, 0, _("Panel"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),panel_menu);
+	
+	gtk_signal_connect_object(GTK_OBJECT(menu),"show",
+				  GTK_SIGNAL_FUNC(show_x_on_panels),
+				  GTK_OBJECT(panel_menu));
 
 	add_menu_separator (menu);
 
@@ -2778,6 +2860,10 @@ create_root_menu(int fake_submenus, int flags)
 					     FALSE);
 		need_separ = TRUE;
 	}
+	if(flags&MAIN_MENU_DEBIAN && !(flags&MAIN_MENU_DEBIAN_SUB)) {
+		root_menu = create_debian_menu(root_menu, fake_submenus, FALSE);
+		need_separ = TRUE;
+	}
 	/*others here*/
 	
 	if(!root_menu)
@@ -2838,6 +2924,22 @@ create_root_menu(int fake_submenus, int flags)
 				   GTK_SIGNAL_FUNC(submenu_to_display),
 				   menuitem);
 	}
+	if(flags&MAIN_MENU_DEBIAN && flags&MAIN_MENU_DEBIAN_SUB) {
+		if(need_separ)
+			add_menu_separator(root_menu);
+		need_separ = FALSE;
+		menu = create_debian_menu(NULL,fake_submenus, TRUE);
+		menuitem = gtk_menu_item_new ();
+		setup_menuitem (menuitem, 0, _("Debian menus"));
+		gtk_menu_append (GTK_MENU (root_menu), menuitem);
+		if(menu) {
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
+						   menu);
+			gtk_signal_connect(GTK_OBJECT(menu),"show",
+					   GTK_SIGNAL_FUNC(submenu_to_display),
+					   menuitem);
+		}
+	}
 	add_special_entries (root_menu, fake_submenus);
 	
 	return root_menu;
@@ -3032,6 +3134,8 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 	GtkWidget *user_sub = gtk_object_get_data(GTK_OBJECT(widget), "user_sub");
 	GtkWidget *redhat_off = gtk_object_get_data(GTK_OBJECT(widget), "redhat_off");
 	GtkWidget *redhat_sub = gtk_object_get_data(GTK_OBJECT(widget), "redhat_sub");
+	GtkWidget *debian_off = gtk_object_get_data(GTK_OBJECT(widget), "debian_off");
+	GtkWidget *debian_sub = gtk_object_get_data(GTK_OBJECT(widget), "debian_sub");
 	GtkWidget *pathentry = gtk_object_get_data(GTK_OBJECT(widget), "path");
 	char *s;
 
@@ -3077,8 +3181,17 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 		menu->main_menu_flags |= MAIN_MENU_REDHAT;
 		menu->main_menu_flags &=~ MAIN_MENU_REDHAT_SUB;
 	}
+	if(GTK_TOGGLE_BUTTON(debian_off)->active)
+		menu->main_menu_flags &=~ (MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB);
+	else if(GTK_TOGGLE_BUTTON(debian_sub)->active)
+		menu->main_menu_flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
+	else {
+		menu->main_menu_flags |= MAIN_MENU_DEBIAN;
+		menu->main_menu_flags &=~ MAIN_MENU_DEBIAN_SUB;
+	}
 
-	gtk_widget_destroy(menu->menu);	
+	if(menu->menu)
+		gtk_widget_destroy(menu->menu);	
 	menu->menu = NULL;
 
 	{
@@ -3267,6 +3380,10 @@ create_properties_dialog(Menu *menu)
 			      _("Red Hat menu (if found): "),"redhat",
 			      menu->main_menu_flags&MAIN_MENU_REDHAT,
 			      menu->main_menu_flags&MAIN_MENU_REDHAT_SUB);
+	add_menu_type_options(GTK_OBJECT(dialog),GTK_TABLE(table),3,
+			      _("Debian menu (if found): "),"debian",
+			      menu->main_menu_flags&MAIN_MENU_DEBIAN,
+			      menu->main_menu_flags&MAIN_MENU_DEBIAN_SUB);
 	
 	f = gtk_frame_new(_("Normal menu"));
 	if(!menu->path || strcmp(menu->path,".")==0)
