@@ -11,12 +11,11 @@
 #include "panel-widget.h"
 #include "button-widget.h"
 #include "panel-util.h"
-#include "gdkextra.h"
 
 GSList *panels=NULL; /*other panels we might want to move the applet to*/
 
 /*define for some debug output*/
-/*#define DEBUG 1*/
+/*#define PANEL_DEBUG 1*/
 
 /*there  can universally be only one applet being dragged since we assume
 we only have one mouse :) */
@@ -456,6 +455,54 @@ panel_widget_is_applet_stuck(PanelWidget *panel, GtkWidget *applet)
 	return FALSE;
 }
 
+static int
+allocate_dirty_child(gpointer data)
+{
+	AppletData *ad = data;
+	PanelWidget *panel;
+	GtkAllocation challoc;
+	GtkRequisition chreq;
+	
+	g_return_val_if_fail(ad != NULL,FALSE);
+	g_return_val_if_fail(ad->applet != NULL,FALSE);
+	g_return_val_if_fail(ad->applet->parent != NULL,FALSE);
+	g_return_val_if_fail(IS_PANEL_WIDGET(ad->applet->parent),FALSE);
+
+	panel = PANEL_WIDGET(ad->applet->parent);
+	
+	if(!ad->dirty)
+		return FALSE;
+	
+	gtk_widget_get_child_requisition(ad->applet,&chreq);
+
+	if(panel->orient == PANEL_HORIZONTAL) {
+		ad->cells = chreq.width + pw_applet_padding;
+		challoc.x = ad->pos;
+		challoc.y = (GTK_WIDGET(panel)->allocation.height - chreq.height) / 2;
+	} else {
+		ad->cells = chreq.height + pw_applet_padding;
+		challoc.x = (GTK_WIDGET(panel)->allocation.width - chreq.width) / 2;
+		challoc.y = ad->pos;
+	}
+	challoc.width = chreq.width;
+	challoc.height = chreq.height;
+	gtk_widget_size_allocate(ad->applet,&challoc);
+
+	gtk_signal_emit(GTK_OBJECT(panel),
+			panel_widget_signals[APPLET_MOVE_SIGNAL],
+			ad->applet);
+
+	return FALSE;
+}
+
+static void
+panel_widget_queue_applet_for_resize(AppletData *ad)
+{
+	ad->dirty = TRUE;
+	g_idle_add(allocate_dirty_child,ad);
+}
+
+
 static void
 panel_widget_switch_applet_right(PanelWidget *panel, GList *list)
 {
@@ -474,20 +521,15 @@ panel_widget_switch_applet_right(PanelWidget *panel, GList *list)
 		nad = nlist->data;
 	if(!nlist || nad->pos > ad->pos+ad->cells) {
 		ad->pos++;
-		gtk_widget_queue_resize(GTK_WIDGET(panel));
+		panel_widget_queue_applet_for_resize(ad);
 		return;
 	}
 	nad->pos = ad->pos;
 	ad->pos = nad->pos+nad->cells;
 	panel->applet_list = my_g_list_swap_prev(panel->applet_list,nlist);
 
-	gtk_signal_emit(GTK_OBJECT(panel),
-			panel_widget_signals[APPLET_MOVE_SIGNAL],
-			ad->applet);
-	gtk_signal_emit(GTK_OBJECT(panel),
-			panel_widget_signals[APPLET_MOVE_SIGNAL],
-			nad->applet);
-	gtk_widget_queue_resize(GTK_WIDGET(panel));
+	panel_widget_queue_applet_for_resize(ad);
+	panel_widget_queue_applet_for_resize(nad);
 }
 
 static void
@@ -508,20 +550,15 @@ panel_widget_switch_applet_left(PanelWidget *panel, GList *list)
 		pad = nlist->data;
 	if(!nlist || pad->pos+pad->cells < ad->pos) {
 		ad->pos--;
-		gtk_widget_queue_resize(GTK_WIDGET(panel));
+		panel_widget_queue_applet_for_resize(ad);
 		return;
 	}
 	ad->pos = pad->pos;
 	pad->pos = ad->pos+ad->cells;
 	panel->applet_list = my_g_list_swap_next(panel->applet_list,nlist);
 
-	gtk_signal_emit(GTK_OBJECT(panel),
-			panel_widget_signals[APPLET_MOVE_SIGNAL],
-			ad->applet);
-	gtk_signal_emit(GTK_OBJECT(panel),
-			panel_widget_signals[APPLET_MOVE_SIGNAL],
-			pad->applet);
-	gtk_widget_queue_resize(GTK_WIDGET(panel));
+	panel_widget_queue_applet_for_resize(ad);
+	panel_widget_queue_applet_for_resize(pad);
 }
 
 static int
@@ -696,88 +733,84 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 }
 
 void
-panel_widget_draw_all(PanelWidget *panel)
+panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 {
 	GList *li;
 	GtkWidget *widget;
 	GdkGC *gc;
+	GdkPixmap *pixmap;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 	
-	if(!panel->pixmap) /*it will get drawn soon anyhow*/
+	widget = GTK_WIDGET(panel);
+
+	if(widget->allocation.width <= 0 ||
+	   widget->allocation.height <= 0 ||
+	   !GTK_WIDGET_DRAWABLE(widget))
 		return;
 
-	widget = GTK_WIDGET(panel);
+	pixmap = gdk_pixmap_new(widget->window,
+				widget->allocation.width,
+				widget->allocation.height,
+				gtk_widget_get_visual(widget)->depth);
 	
-	gc = widget->style->bg_gc[GTK_WIDGET_STATE(widget)];
+	gc = gdk_gc_new(pixmap);
+	gdk_gc_copy(gc,widget->style->bg_gc[GTK_WIDGET_STATE(widget)]);
 
 	if(widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]) {
 		gdk_gc_set_fill(gc, GDK_TILED);
 		gdk_gc_set_tile(gc, widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]);
 	}
 				
-	gdk_draw_rectangle(panel->pixmap,gc, TRUE, 0,0,-1,-1);
+	if(!area)
+		gdk_draw_rectangle(pixmap,gc, TRUE, 0, 0,-1,-1);
+	else
+		gdk_draw_rectangle(pixmap,gc, TRUE, area->x, area->y,
+				   area->width, area->height);
+
+	gdk_gc_unref(gc);
 
-	gdk_gc_set_fill(gc, GDK_SOLID);
-/*	gdk_gc_set_tile(gc, NULL); */
-	
 	for(li = panel->applet_list; li != NULL;
 	    li = g_list_next(li)) {
 		AppletData *ad = li->data;
-		if(IS_BUTTON_WIDGET(ad->applet))
-			button_widget_draw(BUTTON_WIDGET(ad->applet),
-					   panel->pixmap);
+		if(IS_BUTTON_WIDGET(ad->applet) &&
+		   (!area || gtk_widget_intersect(ad->applet, area, NULL)))
+				button_widget_draw(BUTTON_WIDGET(ad->applet),
+						   pixmap);
 	}
-	if(GTK_WIDGET_DRAWABLE(widget))
+	if(!area)
 		gdk_draw_pixmap(widget->window,
 				widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
-				panel->pixmap,
+				pixmap,
 				0,0,0,0,-1,-1);
+	else
+		gdk_draw_pixmap(widget->window,
+				widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
+				pixmap,
+				area->x,area->y,
+				area->x,area->y,
+				area->width,area->height);
+	gdk_pixmap_unref(pixmap);
 }
 
 void
 panel_widget_draw_icon(PanelWidget *panel, ButtonWidget *button)
 {
 	GtkWidget *widget;
-	GtkWidget *applet;
-	GdkGC *gc;
-	
-	g_return_if_fail(panel!=NULL);
-	g_return_if_fail(IS_PANEL_WIDGET(panel));
+	GdkRectangle area;
+
 	g_return_if_fail(button != NULL);
-	g_return_if_fail(IS_BUTTON_WIDGET(button));
-
-	if(!panel->pixmap) /*it will get drawn soon anyhow*/
-		return;
-
-	widget = GTK_WIDGET(panel);
-	applet = GTK_WIDGET(button);
+	g_return_if_fail(GTK_IS_WIDGET(button));
 	
-	gc = widget->style->bg_gc[GTK_WIDGET_STATE(widget)];
-
-	if(widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]) {
-		gdk_gc_set_fill(gc, GDK_TILED);
-		gdk_gc_set_tile(gc, widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)]);
-	}
-				
-	gdk_draw_rectangle(panel->pixmap,gc, TRUE,
-			   applet->allocation.x,applet->allocation.y,
-			   applet->allocation.width,applet->allocation.height);
-
-	gdk_gc_set_fill(gc, GDK_SOLID);
-/*	gdk_gc_set_tile(gc, NULL); */
-
-	button_widget_draw(button, panel->pixmap);
-
-	if(GTK_WIDGET_DRAWABLE(widget))
-		gdk_draw_pixmap(widget->window,
-				widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
-				panel->pixmap,
-				applet->allocation.x,applet->allocation.y,
-				applet->allocation.x,applet->allocation.y,
-				applet->allocation.width,applet->allocation.height);
+	widget = GTK_WIDGET(button);
+	
+	area.x = widget->allocation.x;
+	area.y = widget->allocation.y;
+	area.width = widget->allocation.width;
+	area.height = widget->allocation.height;
 
+	panel_widget_draw_all(panel,&area);
 }
 
 static void
@@ -793,14 +826,9 @@ panel_widget_draw(GtkWidget *widget, GdkRectangle *area)
 
 	if(!GTK_WIDGET_DRAWABLE(widget))
 		return;
-	if(panel->pixmap) {
-		gdk_draw_pixmap(widget->window,
-				widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
-				panel->pixmap,
-				area->x, area->y,
-				area->x, area->y,
-				area->width, area->height);
-	}
+
+	panel_widget_draw_all(panel,area);
+
 	for(li = panel->applet_list;
 	    li != NULL;
 	    li = g_list_next(li)) {
@@ -826,14 +854,9 @@ panel_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 
 	if(!GTK_WIDGET_DRAWABLE(widget))
 		return FALSE;
-	if(panel->pixmap) {
-		gdk_draw_pixmap(widget->window,
-				widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
-				panel->pixmap,
-				event->area.x, event->area.y,
-				event->area.x, event->area.y,
-				event->area.width, event->area.height);
-	}
+
+	panel_widget_draw_all(panel,&event->area);
+
 	for(li = panel->applet_list;
 	    li != NULL;
 	    li = g_list_next(li)) {
@@ -941,6 +964,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			}
 			challoc.width = chreq.width;
 			challoc.height = chreq.height;
+			ad->dirty = FALSE;
 			gtk_widget_size_allocate(ad->applet,&challoc);
 			i += ad->cells;
 		}
@@ -996,6 +1020,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			}
 			challoc.width = chreq.width;
 			challoc.height = chreq.height;
+			ad->dirty = FALSE;
 			gtk_widget_size_allocate(ad->applet,&challoc);
 		}
 	}
@@ -1023,21 +1048,8 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	}
 	g_slist_free(send_move);
 	
-	if (GTK_WIDGET_REALIZED (widget)) {
-		if(!panel->pixmap ||
-		   allocation->width != panel->pixw ||
-		   allocation->height != panel->pixh) {
-			if(panel->pixmap)
-				gdk_pixmap_unref(panel->pixmap);
-			panel->pixw = allocation->width;
-			panel->pixh = allocation->height;
-			panel->pixmap = gdk_pixmap_new(widget->window,
-						       panel->pixw,
-						       panel->pixh,
-						       gtk_widget_get_visual(widget)->depth);
-		}
-		panel_widget_draw_all(panel);
-	}
+	if (GTK_WIDGET_REALIZED (widget))
+		panel_widget_draw_all(panel,NULL);
 }
 
 int
@@ -1098,7 +1110,7 @@ panel_try_to_set_default_back(PanelWidget *panel)
 
 	gtk_widget_set_rc_style(GTK_WIDGET(panel));
 
-	panel_widget_draw_all(panel);
+	panel_widget_draw_all(panel,NULL);
 }
 
 static void
@@ -1126,7 +1138,7 @@ panel_try_to_set_back_color(PanelWidget *panel, GdkColor *color)
 
 	gtk_style_unref(ns);
 
-	panel_widget_draw_all(panel);
+	panel_widget_draw_all(panel,NULL);
 }
 
 void
@@ -1189,7 +1201,7 @@ panel_resize_pixmap(PanelWidget *panel)
 	
 	gtk_style_unref(ns);
 
-	panel_widget_draw_all(panel);
+	panel_widget_draw_all(panel,NULL);
 }
 
 static int
@@ -1216,7 +1228,7 @@ panel_try_to_set_pixmap (PanelWidget *panel, char *pixmap)
 	
 		gtk_style_unref(ns);
 
-		panel_widget_draw_all(panel);
+		panel_widget_draw_all(panel,NULL);
 		return TRUE;
 	}
 
@@ -1265,7 +1277,7 @@ panel_try_to_set_pixmap (PanelWidget *panel, char *pixmap)
 	gtk_style_unref(ns);
 	gtk_object_set_data(GTK_OBJECT(panel),"gdk_image",im);
 	
-	panel_widget_draw_all(panel);
+	panel_widget_draw_all(panel,NULL);
 	return TRUE;
 }
 
@@ -1286,15 +1298,7 @@ panel_widget_realize(GtkWidget *w, gpointer data)
 		panel_try_to_set_back_color(panel, &panel->back_color);
 	}
 
-	if(panel->pixmap)
-		gdk_pixmap_unref(panel->pixmap);
-	panel->pixw = w->allocation.width;
-	panel->pixh = w->allocation.height;
-	panel->pixmap = gdk_pixmap_new(w->window,
-				       panel->pixw,
-				       panel->pixh,
-				       gtk_widget_get_visual(w)->depth);
-	panel_widget_draw_all(panel);
+	panel_widget_draw_all(panel,NULL);
 }
 
 static void
@@ -1371,9 +1375,6 @@ panel_widget_init (PanelWidget *panel)
 			      gtk_widget_get_events(GTK_WIDGET(panel)) |
 			      GDK_BUTTON_RELEASE_MASK);
 	
-	panel->pixmap = NULL;
-	panel->pixw = panel->pixh = -1;
-
 	panel->back_type =PANEL_BACK_NONE;
 	panel->fit_pixmap_bg = FALSE;
 	panel->pixmap_resize_timeout = 0;
@@ -1458,7 +1459,7 @@ panel_widget_new (int packed,
 static void
 _panel_widget_applet_drag_start_no_grab(PanelWidget *panel, GtkWidget *applet)
 {
-#ifdef DEBUG
+#ifdef PANEL_DEBUG
   g_message("Starting drag on a %s at %p\n",
 	    gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
 #endif
@@ -1482,7 +1483,7 @@ panel_widget_applet_drag_start_no_grab(PanelWidget *panel, GtkWidget *applet)
 static void
 _panel_widget_applet_drag_end_no_grab(PanelWidget *panel)
 {
-#ifdef DEBUG
+#ifdef PANEL_DEBUG
 	g_message("Ending drag\n");
 #endif
 	panel->currently_dragged_applet = NULL;
@@ -1501,7 +1502,7 @@ panel_widget_applet_drag_end_no_grab(PanelWidget *panel)
 static void
 _panel_widget_applet_drag_start(PanelWidget *panel, GtkWidget *applet)
 {
-#ifdef DEBUG
+#ifdef PANEL_DEBUG
   g_message("Starting drag [grabbed] on a %s at %p\n",
 	    gtk_type_name(GTK_OBJECT(applet)->klass->type), applet);
 #endif
@@ -1715,10 +1716,7 @@ panel_widget_nice_move(PanelWidget *panel, AppletData *ad, int pos)
 		my_g_list_resort_item(panel->applet_list,ad,
 				      (GCompareFunc)applet_data_compare);
 
-	gtk_signal_emit(GTK_OBJECT(panel),
-			panel_widget_signals[APPLET_MOVE_SIGNAL],
-			ad->applet);
-	gtk_widget_queue_resize(GTK_WIDGET(panel));
+	panel_widget_queue_applet_for_resize(ad);
 }
 
 static int moving_timeout = -1;
@@ -1836,7 +1834,9 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	switch (event->type) {
 		case GDK_BUTTON_PRESS:
 			bevent = (GdkEventButton *) event;
+#ifdef PANEL_DEBUG
 			printf("the appwidget %lX\n",(long)widget);
+#endif
 
 			/* don't propagate this event */
 			if (panel->currently_dragged_applet)
@@ -2071,6 +2071,7 @@ panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_
 		ad->applet = applet;
 		ad->cells = 1;
 		ad->pos = pos;
+		ad->dirty = FALSE;
 		gtk_object_set_data(GTK_OBJECT(applet),PANEL_APPLET_DATA,ad);
 		
 		/*this is a completely new applet, which was not yet bound*/
@@ -2125,6 +2126,9 @@ panel_widget_reparent (PanelWidget *old_panel,
 	ad = gtk_object_get_data(GTK_OBJECT(applet), PANEL_APPLET_DATA);
 	g_return_val_if_fail(ad!=NULL,-1);
 	
+	/*we'll resize both panels anyway*/
+	ad->dirty = FALSE;
+	
 	ad->pos = pos;
 
 	/*reparent applet*/
