@@ -773,6 +773,8 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 				} else
 					gtk_widget_set_sensitive(rem, TRUE);
 				pd->menu_age = 0;
+				
+				show_x_on_panels(panel_menu);
 				gtk_menu_popup(GTK_MENU(panel_menu), NULL, NULL,
 					       global_config.off_panel_popups?
 					        panel_menu_position:NULL,
@@ -1177,18 +1179,3 @@ send_state_change(void)
 					    NULL);
 	}
 }
-
-/* Destroy all panels - called upon exit */
-void
-destroy_all_panels(void)
-{
-	GSList *list = panel_list;
-	while (list) {
-		PanelData *pd = list->data;
-		GSList *tmp = list->next;
-		gtk_widget_destroy (pd->panel);
-		list = tmp;
-	}
-	g_slist_free (panel_list);
-	panel_list = NULL;
-}
