@@ -424,10 +424,11 @@ do_session_save(GnomeClient *client,
 
 	new_args[2] = gnome_config_get_real_path (panel_cfg_path);
 	gnome_client_set_discard_command (client, 3, new_args);
-	
-	/*DEBUG*/printf("Saving to [%s]\n",panel_cfg_path);
+#ifdef PANEL_DEBUG	
+	printf("Saving to [%s]\n",panel_cfg_path);
 
-	/*DEBUG*/printf("Saving session: 1"); fflush(stdout);
+	printf("Saving session: 1"); fflush(stdout);
+#endif
 	if(complete_sync) {
 		GSList *li;
 		for(li=applets;li!=NULL;li=g_slist_next(li))
@@ -437,22 +438,26 @@ do_session_save(GnomeClient *client,
 		for(li = sync_applets; li!=NULL; li=g_slist_next(li))
 			save_applet_configuration(li->data);
 	}
-	/*DEBUG*/printf(" 2"); fflush(stdout);
-
+#ifdef PANEL_DEBUG
+	printf(" 2"); fflush(stdout);
+#endif
 	s = g_concat_dir_and_file(panel_cfg_path,"panel/Config/");
 	gnome_config_push_prefix (s);
 	g_free(s);
 
 	if(complete_sync || sync_applets)
 		gnome_config_set_int ("applet_count", applet_count);
-	/*DEBUG*/printf(" 3"); fflush(stdout);
+#ifdef PANEL_DEBUG
+	printf(" 3"); fflush(stdout);
+#endif
 	if(complete_sync || sync_panels) {
 		num = 1;
 		g_slist_foreach(panel_list, save_panel_configuration,&num);
 		gnome_config_set_int("panel_count",num-1);
 	}
-	/*DEBUG*/printf(" 4"); fflush(stdout);
-
+#ifdef PANEL_DEBUG
+	printf(" 4"); fflush(stdout);
+#endif
 	if(complete_sync || sync_globals) {
 		GString *buf;
 		/*global options*/
@@ -515,8 +520,9 @@ do_session_save(GnomeClient *client,
 
 	gnome_config_pop_prefix ();
 	gnome_config_sync();
-	
-	/*DEBUG*/puts("");
+#ifdef PANEL_DEBUG
+	puts("");
+#endif
 }
 
 void
@@ -638,7 +644,7 @@ panel_really_logout(GtkWidget *w, int button, gpointer data)
 					      GNOME_STOCK_BUTTON_OK,
 					      GNOME_STOCK_BUTTON_CANCEL,
 					      NULL);
-		if(gnome_dialog_run_and_close(dlg)==0) {
+		if(gnome_dialog_run_and_close(GNOME_DIALOG(dlg))==0) {
 			gnome_client_set_restart_style (client,
 							GNOME_RESTART_NEVER);
 			panel_session_save (client, 1, GNOME_SAVE_BOTH, 1,
@@ -731,6 +737,9 @@ load_default_applets(void)
 	/*guess redhat menus*/
 	if(g_file_exists("/etc/X11/wmconfig"))
 		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+	/*guess debian menus*/
+	if (g_file_exists("/etc/menu-methods/gnome"))
+		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
 	load_menu_applet(NULL,flags, panels->data, 0);
 
 	for(i=0;def_launchers[i]!=NULL;i++) {
@@ -841,6 +850,9 @@ init_user_applets(void)
 				/*guess redhat menus*/
 				if(g_file_exists("/etc/X11/wmconfig"))
 					flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+				/*guess debian menus*/
+				if (g_file_exists("/etc/menu-methods/gnome"))
+					flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
 			}
 			load_menu_applet(params,flags,panel,pos);
 			g_free(params);
